%%  RobustnessAsymmetry    Computes the robustness of asymmetry of a quantum state with respect to a given group
%   This function has two required arguments:
%     RHO: a density matrix
%     G: a cell containing a representation of a group G
%
%   [ROA,W] = RobustnessAsymmetry(RHO,G) is the robustness of asymmetry (as
%   defined in [1]) of the quantum state (density matrix) RHO with respect
%   to the group whose representation is given by the multi-dimensional
%   array G. That is, G contains D unitary matrices of the same size as RHO
%   in G(:,:,1), G(:,:,2), ..., G(:,:,D) that represent some group of size
%   D.
%
%   ROA is the numerical value of the robustness of asymmetry, and W is an
%   optimal witness with zeros on its diagonal, all eigenvalues <= 1, and
%   -trace(W*rho) == ROA.
%
%   When G is the set containing the generalized Pauli Z matrices, this
%   function returns the robustness of coherence (see
%   RobustnessCoherence.m).
%
%   References: [1] C. Napoli, T. R. Bromley, M. Cianciaruso, M. Piani, 
%                   N. Johnston, G. Adesso. Robustness of coherence: An
%                   operational and observable measure of quantum
%                   coherence. Preprint submitted to PRL, 2016.
%               [2] M. Piani, M. Cianciaruso, T. R. Bromley, C. Napoli, 
%                   N. Johnston, G. Adesso. Robustness of asymmetry and
%                   coherence of quantum states. Preprint submitted to PRA,
%                   2016.

%   requires: CVX (http://cvxr.com/cvx/)
%   last updated: January 12, 2016

function [RoA,W] = RobustnessAsymmetry(rho,G)

% n is the size of the density matrix
n = length(rho);

% d is the size of the group G
d = length(G);

% The robustness of asymmetry is computed via semidefinite programming.
cvx_begin sdp quiet
    cvx_precision best;
    variable inc_state(n,1);
    variable sig(n,n) hermitian;
    dual variable W;
    
    minimize trace(sig);
    
    subject to
        % tau is the action of the group channel on rho+sig
        tau = zeros(n);
        for j = 1:d
            tau = tau + G(:,:,j)*(rho+sig)*G(:,:,j)';
        end

        sig >= 0;
        W : rho + sig == tau/d;
cvx_end

RoA = real(cvx_optval);
W = full(W);