%%  RobustnessCoherence    Computes the robustness of coherence of a quantum state
%   This function has one required argument:
%     RHO: a pure state vector or a density matrix
%
%   [ROC,W] = RobustnessCoherence(RHO) is the robustness of coherence (as
%   defined in [1,2]) of the quantum state (density matrix) RHO. ROC is the
%   numerical value of the robustness of coherence, and W is an optimal
%   witness with zeros on its diagonal, all eigenvalues <= 1, and
%   -trace(W*rho) == ROC.
%
%   References: [1] C. Napoli, T. R. Bromley, M. Cianciaruso, M. Piani, 
%                   N. Johnston, G. Adesso. Robustness of coherence: An
%                   operational and observable measure of quantum
%                   coherence. Preprint submitted to PRL, 2016.
%               [2] M. Piani, M. Cianciaruso, T. R. Bromley, C. Napoli, 
%                   N. Johnston, G. Adesso. Robustness of asymmetry and
%                   coherence of quantum states. Preprint submitted to PRA,
%                   2016.

%   requires: CVX (http://cvxr.com/cvx/)
%   last updated: January 12, 2016

function [RoC,W] = RobustnessCoherence(rho)

[m,n] = size(rho);

% Let the user enter either a pure state vector or a density matrix.
if(min(m,n) == 1) % it's a pure state vector
    rho = rho(:);
    rho = rho*rho';
    n = max(m,n); % update the size of the state
elseif(m ~= n) % it's neither a pure state vector nor a density matrix
    error('RobustnessCoherence:InvalidDimensions','RHO must be either a vector or a square matrix.');
end

% If the state is pure or single-qubit, we can compute it faster by
% recalling that it is equal to the l1-norm of coherence.
if((n <= 2 || rank(rho) == 1) && nargout <= 1)
    RoC = sum(sum(abs(rho))) - trace(rho);
    return
end

% In general, the robustness of coherence is computed by semidefinite programming
cvx_begin sdp quiet
    cvx_precision best;
    variable inc_state(n,1);
    variable sig(n,n) hermitian;
    dual variable W;
    
    minimize trace(sig);
    
    subject to
        sig >= 0;
        W : rho + sig == diag(inc_state);
cvx_end

RoC = real(cvx_optval);
W = full(W);