%   RobustnessCoherenceLower
%
%   Computes a lower bound to the robustness of coherence of an unknown
%   quantum state based on expectation values of given operators
%
%   This function has two required arguments:
%
%     OBS:  an multi-dimensional array containing observables. Each
%           OBS(:,:,j) is an observable
%
%     EV : a column vector of expectation values for the observables in OBS
%
%   RoClower = RobustnessCoherenceLower(OBS,EV) is a lower bound to the
%   robustness of coherence (as defined in [1,2]) of an unknown quantum
%   state such that EV are the expectation values of the observables OBS
%   with respect to such a state.
%
%   References: [1] C. Napoli, T. R. Bromley, M. Cianciaruso, M. Piani, 
%                   N. Johnston, G. Adesso. Robustness of coherence: An
%                   operational and observable measure of quantum
%                   coherence. Preprint submitted to PRL, 2016.
%               [2] M. Piani, M. Cianciaruso, T. R. Bromley, C. Napoli, 
%                   N. Johnston, G. Adesso. Robustness of asymmetry and
%                   coherence of quantum states. Preprint submitted to PRA,
%                   2016.
%
%   requires: CVX (http://cvxr.com/cvx/)
%   last updated: January 12, 2016


function RoClower = RobustnessCoherenceLower(OBS,EV)
%
%   _n_ is the number of observable considered
%
n = length(EV);
%
%   _d_ is the dimension of the system considered; it is obtained from the
%   size of the observables
%
d = length(OBS(:,:,1));

cvx_begin sdp quiet
%
%   We are going to find the best coherence witness of the form
%   $ W = \sum_i c_i O_i + m I $
%
    variable c(n)
    variable m
    cvx_precision best
%   
%   The target function $-Tr(W\rho) = - [ \sum_ic_io_i + m ]$
%   
    maximize -real(EV'*c+m)
    
    subject to
        W = 0;
        for j = 1:n
            W = W + c(j)*OBS(:,:,j);
        end
        W = W + m*eye(d);
 % 
 %  We impose the conditions such that $W$ is a coherence witness that
 %  provides a bound to the robustenss of coherence
 %
        diag(W) >= 0;
        W <= eye(d);
cvx_end

RoClower = real(cvx_optval);