%   RobustnessCoherenceLowerPhysical
%
%   Computes a lower bound to the robustness of coherence of an unknown
%   quantum state based on expectation values of given operators
%
%   This function has two required arguments:
%
%     OBS:  an multi-dimensional array containing observables. Each
%           OBS(:,:,j) is an observable
%
%     EV : a column vector of expectation values for the observables in OBS
%
%   RoClower = RobustnessCoherenceLowerPhysical(OBS,EV) is a lower bound to the
%   robustness of coherence (as defined in [1,2]) of an unknown quantum
%   state such that EV are the expectation values of the observables OBS
%   with respect to such a state.
%
%   References: [1] C. Napoli, T. R. Bromley, M. Cianciaruso, M. Piani, 
%                   N. Johnston, G. Adesso. Robustness of coherence: An
%                   operational and observable measure of quantum
%                   coherence. Preprint submitted to PRL, 2016.
%               [2] M. Piani, M. Cianciaruso, T. R. Bromley, C. Napoli, 
%                   N. Johnston, G. Adesso. Robustness of asymmetry and
%                   coherence of quantum states. Preprint submitted to PRA,
%                   2016.
%
%   requires: CVX (http://cvxr.com/cvx/)
%   last updated: January 12, 2016

function RoClower = RobustnessCoherenceLowerPhysical(OBS,EV)
%
%   _n_ is the number of observable considered
%
n = length(EV);
%
%   _d_ is the dimension of the system considered; it is obtained from the
%   size of the observables
%
d = length(OBS(:,:,1));

cvx_begin sdp quiet

    variable sig(d,d) hermitian
    variable rho(d,d) hermitian
    variable inc_state(d,1)
    
    cvx_precision best
    
    minimize trace(sig)
    
    subject to
        sig >= 0;
        rho >= 0;
        trace(rho) == 1;
        rho + sig == diag(inc_state);
  
        for j = 1:n
            EV(j)== trace(OBS(:,:,j)*rho);
        end
        
cvx_end

RoClower = real(cvx_optval);